' Kleine Usb -bibliothek:
' Unterprogramme und Interruptroutinen zur Nutzung von USB mit
' ATMEL-AVR-ICs.
' Herstellerspezifische Klasse (usb_lib_vendor_v1_2.bas)
'
' Datum:  07/09/10                Version: 1.2 (16/03/13)
'
' Prozessor: AT90USB162, AT90USB1287, ATmega32u4
' (siehe Zuweisungen im Hauptprogramm)
'
' Quarz: 8 oder 16 MHz  (siehe Zuweisungen)
'
' Informationen zur Funktionsweise:
'
' In den Zuweisungen im Hauptprogramm wird festgelegt mit welcher
' Quarzfrequenz das IC getaktet wird. Auch kann hier ausgewaehlt werden ob ein
' USB-Seriell Baustein emuliert werden soll (FTDI Clone). Das IC selbst wird
' durch Einbinden der entsprechenden inc-Datei im Hauptprogramm ausgewaehlt.
'
' Endpunkt 1 sendet Daten zum PC. Mit der SRAM-Variablen EP1_Flag
' (EP1_flag = 1) wird der Bibliothek im Hauptprogramm mitgeteilt, dass
' Daten vorhanden sind. Die Anzahl der Datenbytes wird der SRAM-Variablen
' Ep1_cnt uebergeben. Die Daten selbst sind in der SRAM-Tabelle Ep1_buf
' abzuspeichern. Die Tabelle und die Variablen sind in der Bibliothek schon
' reserviert und initialisiert (USBINI).
'
' Endpunkt 2 empfaengt Daten vom PC. Dies wird durch das Flag Ep2_flag
' angezeigt.Die Anzahl der empfangenen Daten steht in Ep2_cnt.


' Copyright (c) 2010   Jean-Claude FELTES & Guy WEILER
'                                           weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3


'**********************   USB-Routinen     ************************************

'Teensy 1.0: AT90USB162  16 MHz
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz


#if _chip = 75                          'mega32u4
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 8 MHz
 #if _xtal = 8000000
  Const Pllpre = &H00                   'Bit 2-4 = 0b000 (:1)
 #else
  Const Pllpre = &H10                   'Bit 2-4 = 0b100 (:2)
 #endif
#endif

#if _chip = 78 Or _chip = 74            'mega32u2 oder 90usb162
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 8 MHz
 #if _xtal = 8000000
  Const Pllpre = &H00                   'Bit 2-4 = 0b000 (:1)
 #else
  Const Pllpre = &H04                   'Bit 2-4 = 0b001 (:2)
 #endif
#endif

#if _chip = 70                          '90usb1287
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 2 MHz
 #if _xtal = 8000000
  Const Pllpre = &H0C                   'Bit 2-4 = 0b011 (:4)
 #else
  Const Pllpre = &H14                   'Bit 2-4 = 0b101 (:8)
 #endif
#endif

'USB-Variablen fr Init_Endpoint
Dim Ep_num As Byte                      'Nummer des anzusprechenden EndPoints
Dim Ep_typ As Byte                      'Typ  (0=Contr., 1=Iso, 2=Bulk, 3=Interrupt)
Dim Ep_dir As Bit                       'bertragungsrichtung (0=OUT-EP, 1=IN-EP)
Dim Ep_siz As Byte                      'Anzahl codiert als 2^(Ep_siz+3)
Dim Ep_bk As Byte                       'EP-Bank (0=1 Bank, 1=Banks)

'fr SETUP Paket USB
Dim Su_breq_type As Byte                'bmRequestType
Dim Su_breq As Byte                     'bRequest
Dim Su_val As Word                      'wValue
Dim Su_val_l As Byte At Su_val Overlay
Dim Su_val_h As Byte At Su_val + 1 Overlay
Dim Su_ind As Word                      'wIndex
Dim Su_ind_l As Byte At Su_ind Overlay
Dim Su_ind_h As Byte At Su_ind + 1 Overlay
Dim Su_len As Word                      'wLength
Dim Su_len_l As Byte At Su_len Overlay
Dim Su_len_h As Byte At Su_len + 1 Overlay

'Zwischenvariablen fr USB-Routinen
Dim Btmp As Byte
Dim Btmp2 As Byte
Dim Bctr As Byte
Dim I As Byte

'Buffer fuer die Endpunkte
Dim Ep1_buf(64) As Byte                 'Buffer fuer zu sendende Daten (zum PC)
Dim Ep2_buf(64) As Byte                 'Buffer fuer ankommende Daten (vom PC)

'Flags und Zaehler fuer die Endpunkte
Dim Ep1_flag As Bit                     'Flag Senden zum PC
Dim Ep2_flag As Bit                     'Flag Empfang vom PC, wird automatisch gesetzt
                                                             'falls Daten vorhanden
Dim Ep1_cnt As Byte                     'Anzahl der Datenbytes Senden zum PC
Dim Ep2_cnt As Byte                     'Anzahl der Datenbytes Empfang vom PC

'Falls FTDI:
#if Ftdi = 1
Dim Ff_reset As Byte                    'Ftdi Flag zum Kommando Reset Device
Dim Ff_smod As Byte                     'Ftdi Flag zum Kommando Set Modem Control Reg.
Dim Ff_sbaud As Byte                    'Ftdi Flag zum Kommando Set Baudrate
Dim Ff_sdata As Byte                    'Ftdi Flag zum Kommando Set Data Format

Dim Sm_data As Word                     'Ftdi Daten zum Kommando Set Modem Contr. Register
Dim Sm_data_l As Byte At Sm_data Overlay
Dim Sm_data_h As Byte At Sm_data + 1 Overlay
Dim Sb_data As Word                     'Ftdi Daten zum Kommando Set Baudrate
Dim Sb_data_l As Byte At Sb_data Overlay
Dim Sb_data_h As Byte At Sb_data + 1 Overlay
Dim Sd_data As Word                     'Ftdi Daten zum Kommando Set Data Format
Dim Sd_data_l As Byte At Sd_data Overlay
Dim Sd_data_h As Byte At Sd_data + 1 Overlay
#endif

'USB-Konstanten fr Endpoints
Const Ep0_t = 0                         'EP0-Typ: 0=Control
Const Ep0_d = 0                         'EP0-DIR:0=OUT
Const Ep0_s = 0                         'EP0-Size 2^(0+3)=8 Byte
Const Ep0_b = 0                         '0 = 1 Bank
Const Ep0_fs = 8                        'FIFO size = 8 Byte (Word)

'EP1 = IN EP (PC <- Device)
Const Ep1_t = 2                         'EP1-Typ: Bulk Transfer
Const Ep1_d = 1                         'EP2-DIR: 1=IN  (zum PC)
Const Ep1_s = 3                         'EP1-Size 2^(3+3)=64 Byte
Const Ep1_b = 0                         '0 = 1 Bank
Const Ep1_fs_l = 64                     'FIFO size = 64 Byte (Word)
Const Ep1_fs_h = 0

'EP2 = OUT EP (PC -> Device)
Const Ep2_t = 2                         'EP2-Typ: Bulk Transfer
Const Ep2_d = 0                         'EP2-DIR: 0=OUT (vom PC)
Const Ep2_s = 3                         'EP2-Size 2^(3+3)=64Byte
Const Ep2_b = 0                         '0 = 1 Bank
Const Ep2_fs_l = 64                     'FIFO size = 64 Byte (Word)
Const Ep2_fs_h = 0

'''EP... = ...
'''Hier erweitern

'weitere globale Konstanten

'bei Erweiterung folgende beiden Konstanten anpassen
Const Nr_eps = 2                        'Zahl der benutzten EP (ohne EP0)
Const Totallength = &H09 + &H09 +(&H07 * Nr_eps)

Const Lang_i = 0
Const Manu_i = 1
Const Prod_i = 2
Const Seri_i = 3
Const Intf_i = 2

'-------------------------------------------------------------------------------
'          USB INTERRUPT SERVICE ROUTINEN
'-------------------------------------------------------------------------------

Usb_isr_general:
' ** USB General Interrupt Service Routine
' reagiert nur auf End Of Reset Interrupt EORSTI
If Udint.eorsti = 1 Then
    'General Interrupt sperren
    Udint.eorsti = 0                    'lsche End Of Interrupt Flag
    'Initialisierung  EP0
    Ep_num = 0
    Ep_typ = Ep0_t
    Ep_dir = Ep0_d
    Ep_siz = Ep0_s
    Ep_bk = Ep0_b
    Gosub Usb_init_endpoint
    'erlaube EP COM Interrupts
    Ueienx.rxstpe = 1
End If
Return
'------------------------------------------------------------------------------

Usb_isr_communication:
'** Endpoint/Pipe Communication Interrupt Service Routine

'EP-Nummer durch Bitposition in UEINT  ermitteln
    Select Case Ueint
        Case 1
            'EP0: Enumeration ber EP0
            Uenum = 0                   'EP0  auswhlen
            'Wenn Setup-Paket von PC angekommen, dieses behandeln
            If Ueintx.rxstpi = 1 Then Gosub Usb_ep0_setup
        Case 2
            'EP1: (IN  EP)
            Uenum = 1                   'EP1  auswhlen
            Gosub Usb_ep1_in
        Case 4
            'Ep2: (OUT EP)
            Uenum = 2
            Gosub Usb_ep2_out

            '''hier erweitern
            '''Case 8
            '''EP5: UENUM = 3....

        Case Else
        'andere Flle (die hier nicht implementiert sind)
    End Select
Return
'-------------------------------------------------------------------------------

'-------------------------------------------------------------------------------
'          USB Unterprogramme
'-------------------------------------------------------------------------------
Usb_init_device:
'USB-Aktivierung (Full Speed 12Mbit/s, Interrupts erlauben )

#if _chip = 78 Or _chip = 74            'mega32u2 oder '90usb162
Const Usbc1 = &HA0
#else
Const Usbc1 = &HB0
#endif
#if _chip = 75                          'mega32u4
    Uhwcon = &H01                       'Versorgungsspannung D+, D- ein   (''')
#endif
#if _chip = 70                          '90usb1287
    Usbcon.frzclk = 1                   'Der ATMEL Bootloader nutzt einige Interrupts
    Otgien = 0                          'Deaktiviere diese Interrupts damit sauberer Start
    Udien = 0
    Uhien = 0
    Ueienx = 0
    Upienx = 0
    Uhwcon = &H80                       'Enable Device Modus und Regelkreis zur Versorgung
    Uhwcon = &H81                       'Versorgungsspannung D+, D- ein   (''')
#endif

    Usbcon = Usbc1                      'Sender/Empf., Takt einschalten, VBUS einschalten
    Usbcon.frzclk = 0                   'FRZCLK=0 ntig (kurz), da sonst kein WAKEUP Interrupt
    Usbcon.frzclk = 1                   'FRZCLK=1, Strom sparen

    'PLL starten
    Pllcsr = Pllpre                     'Vorteiler fr Quarz (siehe Zuweisungen
    Pllcsr.plle = 1                     'starte PLL (Enable-Bit = 1)
    Bitwait Pllcsr.plock , Set          'warten bis PLL eingerastet (ca. 100ms)
    Usbcon.frzclk = 0                   'FRZCLK=0: aktiviere Takt

    Udcon.detach = 0                    'Usb attach connect the device
    Udien.eorste = 1                    'Usb enable End of Reset Interrupt (EORSTE=1)

    'Flags, Zhler und Buffer lschen
    Ep1_flag = 0
    Ep2_flag = 0
    Ep1_cnt = 0
    Ep2_cnt = 0

    Dim Index As Integer
    For Index = 1 To 64
        Ep1_buf(index) = 0
    Next Index
    For Index = 1 To 64
         Ep2_buf(index) = 0
    Next Index

#if Ftdi = 1
    Ep1_buf(1) = &H01                   'modem status byte (B0 reserved must be 1)
    Ep1_buf(2) = &H60                   'line status file (B5 Transmitter Holding Reg.
    Ff_reset = 0                        'Flag Reset Device loeschen
    Ff_smod = 0                         'Flag Set Modem Control Reg. loeschen
    Ff_sbaud = 0                        'Flag Set Baudrate loeschen
    Ff_sdata = 0                        'Flag Set Data Format loeschen
#endif

  Enable Interrupts
Return
'-------------------------------------------------------------------------------
Usb_init_endpoint:
'Endpoint initialisieren
'Ep_num, Ep_typ, Ep_dir, Ep_siz, Ep_bk mssen vorher gesetzt sein!

    Uenum = Ep_num                      'EP auswhlen
    Ueconx.epen = 1                     'und aktivieren
    Uecfg0x.0 = Ep_dir                  'Datenrichtung
    Uecfg0x.6 = Ep_typ.0                'Typ in 2 Bits
    Uecfg0x.7 = Ep_typ.1                'Typ in 2 Bits
    'Uecfg1x.alloc = 0
    Uecfg1x.2 = Ep_bk.0                 'FIFO-Bank in 2 Bits
    Uecfg1x.3 = Ep_bk.1                 'FIFO-Bank in 2 Bits
    Uecfg1x.4 = Ep_siz.0
    Uecfg1x.5 = Ep_siz.1                'FIFO-Grsse in 2 Bits
    Uecfg1x.6 = Ep_siz.2                'FIFO-Grsse in 2 Bits
    Uecfg1x.alloc = 1                   'Speicher dem Puffer zuweisen
Return
'------------------------------------------------------------------------------
Usb_ep0_setup:
'Behandle eine SETUP-Anfrage vom PC
'lies Setup-Paket (8 byte) vom PC ins RAM
    Su_breq_type = Uedatx               'bmRequestType
    Su_breq = Uedatx                    'bRequest
    Su_val_l = Uedatx                   'Setup-Value in Word
    Su_val_h = Uedatx
    Su_ind_l = Uedatx                   'Setup-Index in Word
    Su_ind_h = Uedatx
    Su_len_l = Uedatx                   'Setup-Length in Word
    Su_len_h = Uedatx
    'ACK SU-Paket (ACK fr SETUP Packet und FIFO lschen)
    Ueintx.rxstpi = 0
    'Ist es ein Standard Device Request? wenn nicht: Stall und raus
    Btmp = Su_breq_type And &H61        'nur Device oder Interface
    If Btmp = 0 Or Btmp = 1 Then
        'Welche Art Request?
        Select Case Su_breq
            Case 0                      'Get status
                'nur ntig um Programme zufrieden zu stellen
                'die den Status abfragen
                '00-Word zurcksenden (bus powered)
                Uedatx = 0
                Uedatx = 0
                Ueintx.txini = 0        'Daten absenden (ACK) und FIFO lschen
                'Warten bis ZLP vom PC (RXOUTI Flag gesetzt wird)
                Bitwait Ueintx.rxouti , Set
                Ueintx.rxouti = 0       'Flag wieder lschen

            Case 5 :                    'set Address (nur 2 Abschnitte (keine Daten))
                'Gerteadresse (0...127) steht in Low Byte von Su_val
                Udaddr = Su_val_l And &H7F
                Ueintx.txini = 0        'ZLP senden!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                'Adresse aktivieren durch Setzen von Bit 7
                Udaddr.adden = 1

            Case 6 :                    'get descriptor
                Select Case Su_val_h
                    Case 1
                        'Device descriptor
                        Restore Devicedescriptor
                        Read Bctr
                        Restore Devicedescriptor

                    Case 2
                        'Configuration descriptor
                        Bctr = Su_len_l
                        'Manchmal erfragt Windows unsinnig hohe Werte (groesser als 256
                        'Byte). Dadurch steht dann im Lowbyte ev. ein falscher Wert
                        'Es wird hier davon ausgegangen, dass nicht mehr als 256 Byte
                        'angefragt werden knnen.
                        If Su_len > Totallength Or Su_len = 0 Then
                            Bctr = Totallength
                        End If
                        Restore Configurationdescriptor

                    Case 3
                        'String descriptor
                        Btmp = Su_val_l
                        Select Case Btmp
                            Case Lang_i
                                Restore Stringdescriptor0
                                Read Bctr
                                Restore Stringdescriptor0
                            Case Manu_i
                                Restore Stringdescriptor1
                                Read Bctr
                                Restore Stringdescriptor1
                            Case Prod_i
                                Restore Stringdescriptor2
                                Read Bctr
                                Restore Stringdescriptor2
                            Case Seri_i
                                Restore Stringdescriptor3
                                Read Bctr
                                Restore Stringdescriptor3
                        End Select
                    Case Else : Return
                End Select
                'FIFO mit Descriptor fllen und zum PC schicken
                Gosub Usb_send_descriptor

            Case 9 :                    'set configuration
                'Benutzte Endpoints abschalten
                For I = Nr_eps To 1 Step -1       'Rckwrts (siehe Speicherbelegung)
                    Uenum = I
                    Ueconx.epen = 0
                    Uecfg1x.alloc = 0
                Next I

                '----------------------------------------
                'Initialisierung der Endpunkte

                'Initialisierung Endpunkt 1 (IN-EP)
                Ep_num = 1
                Ep_typ = Ep1_t
                Ep_dir = Ep1_d
                Ep_siz = Ep1_s
                Ep_bk = Ep1_b
                Gosub Usb_init_endpoint
                 'NAK INterrupts fr IN Endpunkte erlauben
                'Int. wird von HW gesetzt wenn IN Anfrage von PC mit NAK
                ' beantwortet wurde. Dann wird ein neuer COM INT ausgelst
                Ueienx.nakine = 1

               'Initialisierung Endpunkt2  (OUT-EP)
                Ep_num = 2
                Ep_typ = Ep2_t
                Ep_dir = Ep2_d
                Ep_siz = Ep2_s
                Ep_bk = Ep2_b
                Gosub Usb_init_endpoint
                'RX OUT Interrupts fr OUT Endpunkte erlauben
                'wird von HW gesetzt wenn OUT Daten vorhanden sind
                'Dann wird ein neuer COM INT ausgelst
                Ueienx.rxoute = 1

                ''' hier erweitern
                '----------------------------------------

                Uenum = 0               'EP0
                Ueintx.txini = 0        'sende ZLP (Erfolg, lscht EP-Bank)
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
            Case Else
            Ueconx.stallrq = 1          'Stall (raus)
        End Select
#if Ftdi = 1
    End If
    Btmp = Su_breq_type And &H60
    If Btmp = &H40 Then
        Select Case Su_breq             'Welches FTDI Kommando?
            Case 0                      'Command RESET the Port (OUT, keine Daten)
                Ff_reset = 1            'Setze das Flag fuers Hauptprogramm
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 1                      'Command Set the modem control register (OUT, keine Daten)
                Ff_smod = 1             'Setze das Flag fuers Hauptprogramm
                Sm_data_l = Su_val_l    'Modem Control Daten (word) nach SM_DATA
                Sm_data_h = Su_val_h
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 2                      'Command Set flow control register (OUT, keine Daten)
                                         'nothing to do
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 3                      'Command Set Baudrate (OUT, keine Daten)
                Ff_sbaud = 1            'Setze das Flag fuers Hauptprogramm
                Sb_data_l = Su_val_l    'Baudrate Daten (word) nach SB_DATA
                Sb_data_h = Su_val_h
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 4                      'Command Set Data Format (OUT, keine Daten)
                Ff_sdata = 1            'Setze das Flag fuers Hauptprogramm
                Sd_data_l = Su_val_l    'Datenformat Daten (word) nach SD_DATA
                Sd_data_h = Su_val_h
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 5                      'Command Retrieve current value of modem status register (IN)
                Uedatx = 16             'Standard Wert
                Ueintx.txini = 0        'Daten absenden (ACK) und FIFO lschen
                Bitwait Ueintx.rxouti , Set       'Warten bis ZLP vom PC (RXOUTI Flag gesetzt wird)
                Ueintx.rxouti = 0
            Case 6                      'Command  Set the event character (OUT, keine Daten)
                                        'nothing to do
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 7                      'Command Set the error character (OUT, keine Daten)
                                         'nothing to do
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 9                      ';Command Set latency timer (OUT, keine Daten)
                                        'nothing to do
                Ueintx.txini = 0        'ZLP senden (meldet Erfolg, loescht EP Bank)!
                Bitwait Ueintx.txini , Set       'Warten bis Speicherbank wieder frei
                Ueintx.rxouti = 0
            Case 10                     'Command Retrieve current value of modem status register (IN)
                Uedatx = 16             'Standard Wert
                Ueintx.txini = 0        'Daten absenden (ACK) und FIFO lschen
                Bitwait Ueintx.rxouti , Set       'Warten bis ZLP vom PC (RXOUTI Flag gesetzt wird)
                Ueintx.rxouti = 0       'Flag wieder lschen
            Case Else
            Ueconx.stallrq = 1          'Stall (raus)
        End Select
#endif
    Else
        Ueconx.stallrq = 1              'Stall (raus)
    End If
Return

'-------------------------------------------------------------------------------
Usb_send_descriptor:
'FIFO mit Descriptor fllen und zum PC schicken
    I = 0
    Do
       'Flle FIFO mit Descriptorbyte
        I = I + 1
        Read Btmp
        Uedatx = Btmp
        'immer nach 8 Bytes das Paket absenden und Speicherbank lschen
        Btmp2 = I Mod 8
        If Btmp2 = 0 Then
            Ueintx.txini = 0            'Daten absenden (ACK) und FIFO lschen
            'Warten bis Speicherbank wieder frei oder ACK vom PC (ZLP)
            Do
            Loop Until Ueintx.txini = 1 Or Ueintx.rxouti = 1
        End If
        Bctr = Bctr - 1
        'Wenn Interrupt vom PC (rxouti = 1 PC bricht ab), dann zurck
    Loop Until Ueintx.rxouti = 1 Or Bctr = 0

    'kontrolliere ob ACK(ZLP) vom PC angekommen (schon fertig)
    ' (kommt vor wenn Vielfaches des FIFO-Buffers angefragt waren)
    If Ueintx.rxouti = 1 Then
        Ueintx.rxouti = 0               'Handshake um Interrupt zu besttigen
    Else
        'es sind noch restliche Bytes abzuschicken
        Ueintx.txini = 0                'Paket absenden und Speicherbank lschen
        'Warten bis ACK(ZLP) vom PC angekommen
        Bitwait Ueintx.rxouti , Set     'Warten bis ZLP vom PC (RXOUTI Flag gesetzt wird)Do
        Ueintx.rxouti = 0               'Handshake  um Interrupt zu besttigen
    End If
Return
'-------------------------------------------------------------------------------

Usb_ep1_in:
Print "x";
'Endpunkt 1 OUT    PC erfragt Daten vom Mikrocontroller
If Ueintx.nakini = 1 Then               'Sind Daten vom PC angefragt worden?
    Ueintx.nakini = 0                   'NAKIN Interrupt Flag lschen
    If Ueintx.txini = 1 Then            'Speicherbank frei?
        Ueintx.txini = 0                'lscht EP-Bank
#if Ftdi = 1
        Uedatx = Ep1_buf(1)             'Sende immer die beiden Bytes zum Modem
        Uedatx = Ep1_buf(2)             'und Line Status
#endif
        If Ep1_flag = 1 Then
#if Ftdi = 1
            'Ep1_cnt = Ep1_cnt + 2
            For I = 3 To Ep1_cnt
#else
            For I = 1 To Ep1_cnt
#endif
                Uedatx = Ep1_buf(i)
            Next
        End If
        Ep1_flag = 0
        Ep1_cnt = 0
        Ueintx.fifocon = 0              'erlaube Abschicken der Daten (FIFO freigeben)
    End If
End If
Return

'-------------------------------------------------------------------------------

Usb_ep2_out:
'Endpunkt 2 OUT       Mikrocontroller erhaelt Daten vom PC
If Ueintx.rxouti = 1 Then               'neue Daten vom PC?
    Ueintx.rxouti = 0                   'Datenpaket besttigen (ACK)
    For I_usb = 1 To 64
        If Ueintx.rwal = 1 Then         'Daten im FIFO?
            Ep2_buf(i_usb ) = Uedatx
            Ep2_cnt = I_usb
        Else
             Exit For
       End If
    Next
    Ep2_flag = 1                        'Flag setzen
    Ueintx.fifocon = 0                  'FIFO wieder freigeben
End If
Return

'hier erweitern'

'-------------------------------------------------------------------------------
Devicedescriptor:
Data 18                                 '18 byte
Data &H01                               'bDescriptortype
Data &H10 , &H01                        'Device USB 1.1
Data &H00                               'bDeviceClass, Klassencode = 0
                                         '0 bedeutet: Code definiert auf Interface Level!
Data &H00                               'Bdevicesubclass, Unterklassencode = 0
Data &H00                               'bDeviceProtocol, Protokollcode = 0
Data Ep0_fs                             'FIFO Size
#if Ftdi = 1
                                        'Data &H7D,&H0C       ;idVendor = 0x0C7D Elektor
                                        'Data &H05,&H00       ;idProduct = 0x0005 Elektor
Data &H03 , &H04                        'idVendor = 0x0403, FTDI Code durch usb.org vergeben
Data &H01 , &H60                        'idProduct = 0x6001 FTDI
Data &H00 , &H06                        'bcdDevice = 0x0600, Release Nummer Geraet 6.0
#else
Data &HEB , &H03                        'id_Vendor=Atmel Code
Data &H01 , &H00                        'id_Prod (1 fuer Vendor)
Data &H01 , &H00                        'bcd_Device
#endif
Data Manu_i                             'Index fr stringdescriptor Hersteller
Data Prod_i                             'Index fr Stringdescriptor Produkt
Data Seri_i                             'Index fr Stringdescriptor Seriennummer
Data &H01                               'bNumConfigurations
                                         ' (Anzahl mglicher Konfigurationen)

Configurationdescriptor:
Data 9                                  'Anzahl Bytes
Data &H02                               'bDescriptortype
Data Totallength , 0                    'Lnge Descriptor + alle untergeordneten Descr.
Data &H01                               'Number of Interfaces
Data &H01                               'bConfigurationValue (<>0, sonst unkonfiguriert)
Data 0                                  'iConfiguration (kein Text)
Data &H80                               'bmAttributes  (bus powered, kein remote wakeup)
Data 250                                'MaxPower (250*2mA = 500mA)

'Interface Descriptor
Data &H09                               'bLenghth
Data &H04                               'bDescriptortype
Data 0                                  'bInterfaceNumber
Data 0                                  'bAlternateSettings
Data Nr_eps                             'Number of Endpoints
Data &HFF                               'bInterfaceclass
Data &HFF                               'bInterfaceSubClass
Data &HFF                               'bInterfaceProtocol
Data Intf_i                             'iInterface (0 = kein Text)

'EP1 Descriptor
Data &H07                               'bLength
Data &H05                               'bDescriptortype
Data &H81                               'Endpoint address (bit7=1(IN), bit0-3=EP-Nummer)
Data &H02                               'bmAttributes   (Bulk Transfer)
Data Ep1_fs_l , Ep1_fs_h                'FIFO Size
Data 0                                  'bInterval (Bulk EP, kein Polling)

'EP2 Descriptor
Data &H07                               'bLength
Data &H05                               'bDescriptortype
Data &H02                               'Endpoint address (bit7=0(OUT), bit0-3=EP-Nummer)
Data &H02                               'bmAttributes   (Bulk Transfer)
Data Ep2_fs_l , Ep2_fs_h                'FIFO Size
Data 0                                  'bInterval (Bulk EP, kein Polling)

'''hier erweitern

Stringdescriptor0:
'Language String Descriptor
Data 4                                  'bLength
Data 3                                  'bDescriptortype
Data 9 , 4                              'wLANGID[0] 0409h = USA English

Stringdescriptor1:
'Manufacturer
Data 30 , 3
Data "F" , "E" , "L" , "J" , "C" , " " , "W" , "E" , "I" , "G" , "U" , "." , "L" , "U"

Stringdescriptor2:
'Product String
Data 22 , 3
#if Ftdi = 1
Data "U" , "S" , "B" , "-" , "S" , "E" , "R" , "I" , "A" , "L"
#else
Data "U" , "S" , "B" , "-" , "V" , "E" , "N" , "D" , "O" , "R"
#endif

Stringdescriptor3:
'Serial Number
Data 12 , 3
Data "W" , "V" , "1" , "2" , "b"

'Bei alles Strings: Unicode!!!
' "X" wird als 2 Byte interpretiert!