' Firmware zur Nutzung der AVR-USBs als USB-Serial Converter
' (usb_small_lib_vendor_ftdi_firmware_v1_2_bascom.bas)
'
' USB-Bibliothek (usb_lib_vendor_v1_2.bas)
'
' Datum:  05/05/13                Version: 1.2 (05/05/13)
'
' Prozessor: AT90USB162, AT90USB1287, ATmega32u4
'
' Quarz: 8 oder 16 MHz  (siehe Zuweisungen)
'
' Informationen zur Funktionsweise:


'    Steuerleitungen:
'    PB0 -> DTR
'    PB4 -> RTS
'    Meldeleitungen:
'    PD0 <- DSR
'    PD1 <- CTS
'    PC6 <- RI
'    PC7 <- DCD


' Achtung:  in der BASCOM-DAT-File  m32u4def.dat kontrollieren
' ob UESTA0X = $EE steht , 'wenn nicht: UESTA$ ndern -> UESTA0X
' ob UECFG0X = $EC steht , 'wenn nicht: UECFG$ ndern -> UECFG0X

' Copyright (c) 2013   Jean-Claude FELTES & Guy WEILER
'                                           weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3

'Hier bitte Chip ($regfile) und Quarz ($crystal) durch Auskommentieren
'fr das entsprechende Board auswaehlen!!
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz
'Teensy 1.0: AT90USB162  16 MHz

$regfile = "m32u4def.dat"               '_CHIP = 75
'$regfile = "usb1287.dat"                '_CHIP = 70
'$regfile = "usb162.dat"                 '_CHIP = 74
'Achtung B2.0.7.3 Fehler in Datei USB_EP statt USB_COM!!
'$regfile = "m32u2def.dat"               '_CHIP = 78
$crystal = 16000000
'$crystal = 8000000

$hwstack = 100
$swstack = 100
$framesize = 100

$baud = 38400

'Interrupts
On Usb_gen Usb_isr_general
On Usb_com Usb_isr_communication
On Urxc Isrrx1
On Oc1a Isrtc1

'Hier kann der FTDI Clone mit 1 freigeschaltet werden
Const Ftdi = 1                          'Ftdi = 0 kein Clone

Goto Init                               'Leider noetig damit globale Variablen eingebunden werden
                                         'ohne dass der Code der Isr ausgefhrt wird
$include "usb_lib_vendor_v1_2.bas"      'USB-Bibliothek einbinden
Init:


Dim Blinkflag As Bit                    'Blinken ein/aus
Dim Btmp1 As Byte                       'Zwischenregister
Dim Adcval As Word

'Blink-LED:
Config Pind.6 = Output                  'PB0 -> DTR
'Steuerleitungen:
Config Pinb.0 = Output                  'PB0 -> DTR
Config Pinb.4 = Output                  'PB4 -> RTS
Dtr Alias Portb.0
Rts Alias Portb.4
'Meldeleitungen:
Config Pind.0 = Input                   'PD0 <- DSR
Config Pind.1 = Input                   'PD1 <- CTS
Config Pinc.6 = Input                   'PC6 <- RI
Config Pinc.7 = Input                   'PC7 <- DCD
Dsr Alias Pind.0
Cts Alias Pind.1
Ri Alias Pinc.6
Dcd Alias Pinc.7
Portd.0 = 1                             'Pull-Ups
Portd.1 = 1
Portc.6 = 1
Portc.7 = 1

Config Clockdiv = 1

'Config Timer1 = Timer
Tccr1a = 0
Tccr1b = &H0D
#if _chip = 70                          'At90usb1287
Ocr1a = 3905
#else
Ocr1a = 7811
#endif

Gosub Usb_init_device                   'USB-Aktivierung

Enable Oc1a                             'Timer Interrupt erlauben
Enable Urxc                             'USART Empfanfs Interrupt erlauben

'-------------------------------------------------------------------------------

Do
 'Print "Test";
 'Disable Interrupts

  'Falls Daten ankommen (Ep2_flag = 1) wird die Anzahl der Bytes ermittelt
  'und sie werden ber die serielle Schnittstelle versendet
  If Ep2_flag = 1 Then
      For I = 1 To Ep2_cnt
          Print chr(Ep2_buf(i));
      Next
      Ep2_flag = 0
  End If

  'Meldeleitungen abfragen und Information weitergeben (Ep1_buf[0])
  'Meldeleitungen sind Nullaktiv!
  If Dsr = 0 Then
     Ep1_buf(1).5 = 1
  Else
     Ep1_buf(1).5 = 0
  End If
  If Cts = 0 Then
     Ep1_buf(1).4 = 1
  Else
     Ep1_buf(1).4 = 0
  End If
  If Ri = 0 Then
     Ep1_buf(1).6 = 1
  Else
      Ep1_buf(1).6 = 0
  End If
  If Dcd = 0 Then
     Ep1_buf(1).7 = 1
  Else
      Ep1_buf(1).7 = 0
  End If
  'Enable Interrupts

  'Falls Kommando RESET dann deaktiviere DTR und RTS
  If Ff_reset = 1 Then
     Dtr = 1                            'DTR und RTS inaktiv (1)
     Rts = 1
     Ff_reset = 0
  End If
  'Falls Kommando Set Modem aktiviere bzw. deaktiviere DTR und RTS
  'B0 fuer DTR, B1 fuer RTS
  If Ff_smod = 1 Then
     If Sm_data_l.0 = 1 Then
        Dtr = 1
     Else
         Dtr = 0
     End If
     If Sm_data_l.1 = 1 Then
        Rts = 1
     Else
         Rts = 0
     End If
     Ff_smod = 0
  End If


   Waitms 3

Loop


'Interruptroutine zum Empfang eines Zeichens
Isrrx1:
       If Ep1_flag = 0 Then
          Ep1_buf(3) = Udr
          Ep1_cnt = 3
       Else
          Ep1_cnt = Ep1_cnt + 1
          Ep1_buf(ep1_cnt) = Udr
       End If
       Ep1_flag = 1
Return


'Interruptroutine Timer1
Isrtc1:
       Toggle Portd.6
Return